@@ -6,7 +6,7 @@
   "If you sit down at a poker game and don't see a sucker, 
   get up. You're the sucker."
 
-  (C) Copyright 2000-2004 by Michal Zalewski <lcamtuf@coredump.cx>
+  (C) Copyright 2000-2006 by Michal Zalewski <lcamtuf@coredump.cx>
 
   WIN32 port (C) Copyright 2003-2004 by Michael A. Davis <mike@datanerds.net>
              (C) Copyright 2003-2004 by Kirby Kuehl <kkuehl@cisco.com>
@@ -36,7 +36,12 @@
 #include <pcap.h>
 #include <signal.h>
 
-#include <net/bpf.h>
+#ifdef USE_BPF
+#include USE_BPF
+#else
+#include <pcap-bpf.h>
+#endif /* ^USE_BPF */
+
 #include <time.h>
 #include <ctype.h>
 
@@ -49,6 +54,7 @@
 #include "tos.h"
 #include "fpentry.h"
 #include "p0f-query.h"
+#include "crc32.h"
 
 #ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
@@ -107,6 +113,7 @@ static _u8 *config_file,
 
 static _u32 query_cache = DEFAULT_QUERY_CACHE;
 static _s32 masq_thres;
+static _s32 capture_timeout = 1;
 
 static _u8 no_extra,
            find_masq,
@@ -130,12 +137,17 @@ static _u8 no_extra,
            full_dump,
            use_fuzzy,
            use_vlan,
-           payload_dump;
+           payload_dump,
+           port0_wild;
 
 static pcap_t *pt;
 static struct bpf_program flt;
 
-static _u32 st_time,pkcnt;
+/* Exports for p0f statistics */
+_u32 packet_count;
+_u8  operating_mode;
+_u32 st_time;
+_u32 file_cksum;
 
 
 static void die_nicely(_s32 sig) {
@@ -143,8 +155,8 @@ static void die_nicely(_s32 sig) {
   if (pt) pcap_close(pt);
   if (dumper) pcap_dump_close(dumper);
 
-  if (!no_banner && pkcnt) {
-    float r = pkcnt * 60;
+  if (!no_banner && packet_count) {
+    float r = packet_count * 60;
     
     r /= (time(0) - st_time);
 
@@ -167,6 +179,10 @@ static void set_header_len(_u32 type) {
     case DLT_SLIP:
     case DLT_RAW:  break;
 
+#ifdef DLT_C_HDLC
+    case DLT_C_HDLC:
+#endif
+
     case DLT_NULL: header_len=4; break;
 
     case DLT_EN10MB: header_len=14; break;
@@ -222,11 +238,11 @@ static void usage(_u8* name) {
           "\nUsage: %s [ -f file ] [ -i device ] [ -s file ] [ -o file ]\n"
 
 #ifndef WIN32
-          "       [ -w file ] [ -Q sock ] [ -u user ] [ -FXVONDUKASCMRqtpvdlr ]\n"
-          "       [ -c size ] [ -T nn ] [ 'filter rule' ]\n"
+          "       [ -w file ] [ -Q sock [ -0 ] ] [ -u user ] [ -FXVNDUKASCMROqtpvdlrx ]\n"
+          "       [ -c size ] [ -T nn ] [ -e nn ] [ 'filter rule' ]\n"
 #else
-		  "       [ -w file ] [ -XVNDUKASCMLRqtpdlrx ]\n"
-		  "       [ -c size]  [ -T nn ] [ 'filter rule' ]\n"
+		  "       [ -w file ] [ -FXVNDUKASCMLROqtpvdlrx ]\n"
+		  "       [ -c size]  [ -T nn ] [ -e nn ] [ 'filter rule' ]\n"
 #endif /* ^WIN32 */
 
           "  -f file   - read fingerprints from file\n"
@@ -237,7 +253,9 @@ static void usage(_u8* name) {
 #ifndef WIN32
           "  -u user   - chroot and setuid to this user\n"
           "  -Q sock   - listen on local socket for queries\n"
+          "  -0        - make src port 0 a wildcard (in query mode)\n"
 #endif /* !WIN32 */
+          "  -e ms     - pcap capture timeout in milliseconds (default: 1)\n"
           "  -c size   - cache size for -Q and -M options\n"
           "  -M        - run masquerade detection\n"
           "  -T nn     - set masquerade detection threshold (1-200)\n"
@@ -483,7 +501,6 @@ reloop:
 }
 
 
-
 static void load_config(_u8* file) {
   _u32 ln=0;
   _u8 buf[MAXLINE];
@@ -509,6 +526,8 @@ static void load_config(_u8* file) {
     _u8* gptr = genre;
     _u32 t,d,s;
     struct fp_entry* e;
+    
+    file_cksum ^= crc32(buf, strlen(buf));
       
     ln++;
 
@@ -926,9 +945,7 @@ static void dump_payload(_u8* data,_u16 dlen) {
 }
 
   
-
-  
-
+_u32 matched_packets;
 
 static inline void find_match(_u16 tot,_u8 df,_u8 ttl,_u16 wss,_u32 src,
                        _u32 dst,_u16 sp,_u16 dp,_u8 ocnt,_u8* op,_u16 mss,
@@ -1023,6 +1040,8 @@ re_lookup:
 continue_fuzzy:    
     
     /* Match! */
+    
+    matched_packets++;
 
     if (mss & wss) {
       if (p->wsize_mod == MOD_MSS) {
@@ -1099,6 +1118,7 @@ continue_fuzzy:
                             tstamp ? tstamp / 360000 : -1);
      a=(_u8*)&src;
      if (sc > masq_thres) {
+       if (add_timestamp) put_date(pts);
        printf(">> Masquerade at %u.%u.%u.%u%s: indicators at %d%%.",
               a[0],a[1],a[2],a[3],grab_name(a),sc);
        if (!mode_oneline) putchar('\n'); else printf(" -- ");
@@ -1245,7 +1265,7 @@ static void parse(_u8* none, struct pcap_pkthdr *pph, _u8* packet) {
   _u32   tstamp = 0;
   _u32   quirks = 0;
 
-  pkcnt++;
+  packet_count++;
 
   if (dumper) pcap_dump((_u8*)dumper,pph,packet);
 
@@ -1474,6 +1494,7 @@ end_parsing:
 }
 
 
+
 int main(int argc,char** argv) {
   _u8 buf[MAXLINE*4];
   _s32 r;
@@ -1483,14 +1504,14 @@ int main(int argc,char** argv) {
   _u8 ebuf[PCAP_ERRBUF_SIZE];
   pcap_if_t *alldevs, *d;
   _s32 adapter, i;
-  while ((r = getopt(argc, argv, "f:i:s:o:w:c:T:XONVFDxKUqvtpArRlSdCLM")) != -1)
+  while ((r = getopt(argc, argv, "f:i:s:o:w:c:T:e:XONVFDxKUqvtpArRlSdCLM")) != -1)
 #else
   _s32 lsock=0;
 
   if (getuid() != geteuid())
     fatal("This program is not intended to be setuid.\n");
   
-  while ((r = getopt(argc, argv, "f:i:s:o:Q:u:w:c:T:XOFNVDxKUqtRpvArlSdCM")) != -1) 
+  while ((r = getopt(argc, argv, "f:i:s:o:Q:u:w:c:e:T:XOFNVDxKUqtRpvArlSdCM0")) != -1) 
 #endif /* ^WIN32 */
 
     switch (r) {
@@ -1514,9 +1535,14 @@ int main(int argc,char** argv) {
       case 'T': masq_thres = atoi(optarg);
                 if (masq_thres <= 0 || masq_thres > 200) fatal("Invalid -T value.\n");
                 break;
+		
+      case 'e': capture_timeout = atoi(optarg);
+                if (capture_timeout <= 0 ||capture_timeout > 10000) fatal("Invalid -e value.\n");
+                break;
 
 #ifndef WIN32
       case 'Q': use_cache  = optarg; break;
+      case '0': port0_wild = 1;      break;
       case 'u': set_user   = optarg; break;
 #endif /* !WIN32 */
 
@@ -1571,6 +1597,8 @@ int main(int argc,char** argv) {
 
       default: usage(argv[0]);
     }
+    
+  if (!use_cache && port0_wild) fatal("-0 requires -Q (query mode).\n");
 
   if (use_logfile && !add_timestamp) add_timestamp = 1;
 
@@ -1614,13 +1642,13 @@ int main(int argc,char** argv) {
   load_config(config_file);
 
   if (argv[optind] && *(argv[optind])) {
-    sprintf(buf,"(%s) and (%3000s)",use_rule,argv[optind]);
+    sprintf(buf,"(%s) and (%.3000s)",use_rule,argv[optind]);
     use_rule = buf;
   } 
 
   if (use_vlan) {
     _u8* x = strdup(use_rule);
-    sprintf(buf,"(%1000s) or (vlan and (%1000s))",x,x);
+    sprintf(buf,"(%.1000s) or (vlan and (%.1000s))",x,x);
     free(x);
     use_rule = buf;
   }
@@ -1638,6 +1666,7 @@ int main(int argc,char** argv) {
     lsock = socket(PF_UNIX,SOCK_STREAM,0);
     if (lsock < 0) pfatal("socket");
 
+    memset(&x,0,sizeof(x));
     x.sun_family = AF_UNIX;
     strncpy(x.sun_path,use_cache,63);
     unlink(use_cache);
@@ -1676,7 +1705,7 @@ int main(int argc,char** argv) {
        course, the documentation sucks, and if you use the timeout
        of zero, things will break. */
     
-    if (!(pt=pcap_open_live(use_iface,PACKET_SNAPLEN,use_promisc,1,errbuf))) 
+    if (!(pt=pcap_open_live(use_iface,PACKET_SNAPLEN,use_promisc,capture_timeout,errbuf))) 
       fatal("pcap_open_live failed: %s\n",errbuf);
   }
 
@@ -1690,10 +1719,10 @@ int main(int argc,char** argv) {
     }
 
   if (!no_banner) {
-    debug("p0f: listening (%s) on '%s', %d sigs (%d generic), rule: '%s'.\n",
+    debug("p0f: listening (%s) on '%s', %d sigs (%d generic, cksum %08X), rule: '%s'.\n",
           ack_mode ? "SYN+ACK" : rst_mode ? "RST+" :
           open_mode ? "OPEN" : "SYN",
-          use_dump?use_dump:use_iface,sigcnt,gencnt,
+          use_dump?use_dump:use_iface,sigcnt,gencnt,file_cksum,
           argv[optind]?argv[optind]:"all");
 
     if (use_cache) debug("[*] Accepting queries at socket %s (timeout: %d s).\n",use_cache,QUERY_TIMEOUT);
@@ -1709,6 +1738,12 @@ int main(int argc,char** argv) {
       exit(1);
     }
   }
+  
+  /* For p0f statistics */
+  if (ack_mode) operating_mode = 'A'; 
+  else if (rst_mode) operating_mode = 'R';
+  else if (open_mode) operating_mode = 'O';
+  else operating_mode = 'S';
 
 #ifndef WIN32
 
@@ -1717,8 +1752,14 @@ int main(int argc,char** argv) {
 
     if (geteuid()) fatal("only root can use -u.\n");
 
+    tzset();
+
     pw = getpwnam(set_user);
     if (!pw) fatal("user %s not found.\n",set_user);
+    
+    if (use_cache && chown(use_cache,pw->pw_uid,pw->pw_gid)) 
+      debug("[!] Failed to set ownership of query socket.");
+ 
     if (chdir(pw->pw_dir)) pfatal(pw->pw_dir);
     if (chroot(pw->pw_dir)) pfatal("chroot");
     chdir("/");
@@ -1739,6 +1780,7 @@ int main(int argc,char** argv) {
 #ifndef WIN32
     _s32 f;
     struct timeval tv;
+    FILE* pid_fd;
     fflush(0);
     f = fork();
     if (f<0) pfatal("fork() failed");
@@ -1748,6 +1790,12 @@ int main(int argc,char** argv) {
     chdir("/");
     setsid();
     signal(SIGHUP,SIG_IGN);
+    
+    if ((pid_fd = fopen(PID_PATH, "w"))) {
+      fprintf(pid_fd, "%d", getpid());
+      fclose(pid_fd);
+    }
+    
     printf("--- p0f " VER " resuming operations at ");
     gettimeofday(&tv, (struct timezone*)0);
     put_date(tv);
@@ -1806,7 +1854,7 @@ int main(int argc,char** argv) {
 
         if (select(c+1,&f,0,&f,&tv)>0)
           if (recv(c,&q,sizeof(q),MSG_NOSIGNAL) == sizeof(q)) 
-            p0f_handlequery(c,&q);
+            p0f_handlequery(c,&q,port0_wild);
 
         shutdown(c,2); 
         close(c);
